/**********************************************************************************************************************
 * Censor Types
 **********************************************************************************************************************/
class CensorTypeConfiguration {

    constructor(name, scale = 1.0) {
        this._name = name;
        this._scale = scale;
    }

    get name() {
        return this._name;
    }

    set name(value) {
        this._name = value;
    }

    get scale() {
        return this._scale;
    }

    set scale(value) {
        this._scale = value;
    }

}

class BarCensorTypeConfiguration extends CensorTypeConfiguration{

    constructor(name, scale, color) {
        super(name, scale);
        this._color = color;
    }

    get color() {
        return this._color;
    }

    set color(value) {
        this._color = value;
    }
}

class BlurCensorTypeConfiguration extends CensorTypeConfiguration{

    constructor(name, scale, blur_strength, blur_radius, mask_scale) {
        super(name, scale);
        this._blur_radius = blur_radius;
        this._mask_scale = mask_scale;
        this._blur_strength = blur_strength;
    }

    get blur_radius() {
        return this._blur_radius;
    }

    set blur_radius(value) {
        this._blur_radius = value;
    }

    get blur_strength() {
        return this._blur_strength;
    }

    set blur_strength(value) {
        this._blur_strength = value;
    }

    get mask_scale() {
        return this._mask_scale;
    }

    set mask_scale(value) {
        this._mask_scale = value;
    }

}

class PixelCensorTypeConfiguration extends CensorTypeConfiguration{

    constructor(name, geometry, scale, distance, pixel_size_mode, pixel_size) {
        super(name, scale);
        this._distance = distance;
        this._geometry = geometry;
        this._pixel_size_mode = pixel_size_mode;
        this._pixel_size = pixel_size;

    }

    get distance() {
        return this._distance;
    }

    set distance(value) {
        this._distance = value;
    }


    get pixel_size_mode() {
        return this._pixel_size_mode;
    }

    set pixel_size_mode(value) {
        this._pixel_size_mode = value;
    }

    get pixel_size() {
        return this._pixel_size;
    }

    set pixel_size(value) {
        this._pixel_size = value;
    }

    set geometry(value) {
        this._geometry = value;
    }
}

class GlitchCensorTypeConfiguration extends CensorTypeConfiguration{

    constructor(name, scale, intensity, shift_horizontal, shift_vertical) {
        super(name, scale);
        this.intensity = intensity;
        this.shift_horizontal = shift_horizontal;
        this.shift_vertical = shift_vertical;

        /*
        this.r_channel_intensity = 0;
        this.g_channel_intensity = 0;
        this.b_channel_intensity = 0;
        this.r_negate_intensity = 0;
        this.g_negate_intensity = 0;
        this.b_negate_intensity = 0;
        this.shift_horizontal = false;
        this.shift_vertical = false;
        this.h_bar_size_range = [0, 0];
        this.v_bar_size_range = [0, 0];
        this.shift_intensity_range = [0, 0];
        */
    }
        /*
        set_channel_intensities(value) {
            this.r_channel_intensity = value;
            this.g_channel_intensity = value;
            this.b_channel_intensity = value;
        }

        set_negation_intensities(value) {
            this.r_negate_intensity = value;
            this.g_negate_intensity = value;
            this.b_negate_intensity = value;
        }

        set_all_intensities(value) {
            this.set_channel_intensities(value);
            this.set_negation_intensities(value);
        }
        */
}

/**********************************************************************************************************************
 * File Types
 **********************************************************************************************************************/

class FileTypeConfiguration {

    constructor(name) {
        this._name = name;
    }

    get name() {
        return this._name;
    }

    set name(value) {
        this._name = value;
    }


}

class GIFTypeConfiguration extends FileTypeConfiguration{

    constructor(name, memory_method, thumbnails) {
        super(name);
        this._memory_method = memory_method;
        this._thumbnails = thumbnails;
    }

    get memory_method() {
        return this._memory_method;
    }

    set memory_method(value) {
        this._memory_method = value;
    }

    get thumbnails() {
        return this._thumbnails;
    }

    set thumbnails(value) {
        this._thumbnails = value;
    }
}

/**
 * Repairs missing configuration variables.
 * Missing variables can often occur after version updates, when new ones got introduced.
 */
function repairConfigurations(){

    if(bar_configuration._color == null || typeof bar_configuration._color === "undefined"){
        bar_configuration._color = default_bar_configuration._color;
    }
    if(bar_configuration._scale == null || typeof bar_configuration._scale === "undefined"){
        bar_configuration._scale = default_bar_configuration._scale;
    }

    if(blur_configuration._scale == null || typeof blur_configuration._scale === "undefined"){
        blur_configuration._scale = default_blur_configuration._scale;
    }
    if(blur_configuration._blur_radius == null || typeof blur_configuration._blur_radius === "undefined"){
        blur_configuration._blur_radius = default_blur_configuration._blur_radius;
    }
    if(blur_configuration._mask_scale == null || typeof blur_configuration._mask_scale === "undefined"){
        blur_configuration._mask_scale = default_blur_configuration._mask_scale;
    }
    if(blur_configuration._blur_strength == null || typeof blur_configuration._blur_strength === "undefined"){
        blur_configuration._blur_strength = default_blur_configuration._blur_strength;
    }

    if(pixel_configuration._scale == null || typeof pixel_configuration._distance === "undefined"){
        pixel_configuration._scale = default_pixel_configuration._scale;
    }
    if(pixel_configuration._distance == null || typeof pixel_configuration._distance === "undefined"){
        pixel_configuration._distance = default_pixel_configuration._distance;
    }
    if(pixel_configuration._pixel_size_mode == null || typeof pixel_configuration._pixel_size_mode === "undefined"){
        pixel_configuration._pixel_size_mode = default_pixel_configuration._pixel_size_mode;
    }
    if(pixel_configuration._pixel_size == null || typeof pixel_configuration._pixel_size === "undefined"){
        pixel_configuration._pixel_size = default_pixel_configuration._pixel_size;
    }
    if(pixel_configuration._geometry == null || typeof pixel_configuration._geometry === "undefined"){
        pixel_configuration._geometry = default_pixel_configuration._geometry;
    }

    if(gif_configuration._memory_method == null || typeof pixel_configuration._memory_method === "undefined"){
        gif_configuration._memory_method = default_gif_configuration._memory_method;
    }
    if(gif_configuration._thumbnails == null || typeof pixel_configuration._thumbnails === "undefined"){
        gif_configuration._thumbnails = default_gif_configuration._thumbnails;
    }

    browser.storage.sync.set({
        bar_configuration: bar_configuration,
        pixel_configuration: pixel_configuration,
        blur_configuration: blur_configuration,
        gif_configuration: gif_configuration,
    });

}

function hexConverter(hex) {
    return hex.replace('#','0xFF');
}

const default_bar_configuration = new BarCensorTypeConfiguration("bar", 1.0, '#000000');
var bar_configuration = default_bar_configuration;

const default_pixel_configuration = new PixelCensorTypeConfiguration("pixel", 0,1.0, 0, 0, 2);
var pixel_configuration = default_pixel_configuration;

const default_blur_configuration = new BlurCensorTypeConfiguration("blur", 2.5, 20, 2, 0.8);
var blur_configuration = default_blur_configuration;

const default_glitch_configuration = new GlitchCensorTypeConfiguration("glitch", 1, 1, true, true);
var glitch_configuration = default_glitch_configuration;


const default_gif_configuration = new GIFTypeConfiguration("gif", 0, true);
var gif_configuration = default_gif_configuration;




