let lastMenuInstanceId = 0;
let nextMenuInstanceId = 1;

browser.contextMenus.create({
    id: "puryFi-censor",
    title: "PuryFi",
    contexts: ["image"],
    icons: {
        "16": "icons/icon_16.png",
        "32": "icons/icon_32.png"
    }
}, onCreated);

browser.contextMenus.create({
    id: "puryFi-pixelate",
    title: "Pixelate",
    contexts: ["image"],
    parentId: "puryFi-censor",
}, onCreated);

browser.contextMenus.create({
    id: "puryFi-black",
    title: "Black Bar",
    contexts: ["image"],
    parentId: "puryFi-censor",
}, onCreated);

browser.contextMenus.create({
    id: "puryFi-blur",
    title: "Blur",
    contexts: ["image"],
    parentId: "puryFi-censor",
}, onCreated);

browser.contextMenus.create({
    id: "puryFi-box",
    title: "Box",
    contexts: ["image"],
    parentId: "puryFi-censor",
}, onCreated);

browser.contextMenus.create({
    id: "puryFi-glitch",
    title: "Glitch",
    contexts: ["image"],
    parentId: "puryFi-censor",
}, onCreated);

function onCreated() {
    if (browser.runtime.lastError) {
        console.log(`Error: ${browser.runtime.lastError}`);
    } else {
        //console.log("Item created successfully");
    }
}

browser.contextMenus.onShown.addListener(async function(info, tab) {
    let menuInstanceId = nextMenuInstanceId++;
    lastMenuInstanceId = menuInstanceId;

    // Call an async function
    let gif = false;
    let ct = await getCT(info.srcUrl);
    if(ct === "image/gif"){
        gif = true;
    }
    // After completing the async operation, check whether the menu is still shown.
    if (menuInstanceId !== lastMenuInstanceId) {
        return; // Menu was closed and shown again.
    }
    // Now use menus.create/update + menus.refresh.
    if(gif){
        browser.contextMenus.update("puryFi-censor", {
            enabled: user != null && user.permissions.permission_file_type_gif <= user.patreon_tier,
        });
    }else{
        browser.contextMenus.update("puryFi-censor", {
            enabled: true,
        });
    }
    browser.contextMenus.refresh();

});


async function getCT(url) {
    return new Promise(resolve => {
        let xhr1 = new XMLHttpRequest();
        let ct = "";
        function hand () {
            ct = this.getResponseHeader('content-type');
        }
        xhr1.onreadystatechange = hand;
        xhr1.onload = function() {
            let reader = new FileReader();
            reader.onloadend = function() {
                resolve(ct);
            }
            reader.readAsArrayBuffer(xhr1.response);
        };
        xhr1.open('GET', url);
        xhr1.responseType = 'blob';
        xhr1.send();
    });
}

browser.contextMenus.onClicked.addListener(function(info, tab) {
    switch (info.menuItemId) {
        case "puryFi-pixelate":
            handleContextCall(info, tab,'pixel')
            break;
        case "puryFi-black":
            handleContextCall(info, tab,'black')
            break;
        case "puryFi-box":
            handleContextCall(info, tab,'box')
            break;
        case "puryFi-blur":
            handleContextCall(info, tab,'blur')
            break;
        case "puryFi-glitch":
            handleContextCall(info, tab,'glitch')
            break;
    }
});

function handleContextCall(info, tab, type){
    toDataURL(info.pageUrl, function (img_data, content_type) {

            if(processableContentType(content_type,file_types,gif_configuration)){
                //Add pageURL to the forcedRequestList and refresh the page
                context_requests.push({
                    censor_type: type,
                    url: info.pageUrl,
                    originalUrl: info.pageUrl}
                );
                browser.tabs.reload({bypassCache: true});
            }else{
                // Load image from image url and message content script to replace element.
                browser.tabs.sendMessage(
                    tab.id,
                    {reload: info.srcUrl,}
                ).then(response => {
                    context_requests.push({
                            censor_type: type,
                            url: response.add_context_request,
                            originalUrl: response.add_context_request_original_url

                        }
                    );
                });

            }


    });
}

function handleFrameMessage(request, sender, sendResponse) {
    if(request.frame){
        let tempCanvas = document.createElement('canvas');
        let tempCtx = tempCanvas.getContext('2d');
        tempCanvas.width = request.width;
        tempCanvas.height = request.height;
        tempCtx.putImageData(request.frame, 0,0, 0, 0, request.width, request.height);
        const fromPixels = tf.browser.fromPixels(tempCanvas);
        const resized = tf.image.resizeBilinear(fromPixels, [modelWeight, modelHeight]);
        const divved = resized.div(255.0);
        const input = divved.expandDims(0);
        model.executeAsync(input).then(res => {
            const [boxes, scores, classes, valid_detections] = res;
            let valid_detections_sync =  valid_detections.dataSync();
            let classes_sync =  classes.dataSync();
            let boxes_sync =  boxes.dataSync();
            let scores_sync =  scores.dataSync();
            valid_detections.dispose();
            classes.dispose();
            scores.dispose();
            boxes.dispose();
            sendResponse({
                valid_detections: valid_detections_sync,
                classes: classes_sync,
                boxes: boxes_sync,
                scores: scores_sync,
            });
        }).catch(e => {
            log(e.message)
            return;
        }).finally(fin => {
            resized.dispose();
            divved.dispose();
            input.dispose();
            fromPixels.dispose();
            return;
        });


        return true;
    }

}

browser.runtime.onMessage.addListener(handleFrameMessage);
